<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Api\ApiController;
use App\Http\Services\Web\SeminarSignUpService;
use App\Http\Requests\Api\SeminarSignUp\StoreRequest;

use Log;

class SeminarSignUpController extends ApiController
{
    
    private $seminarSignUpSv;
    
    public function __construct()
    {
        $this->seminarSignUpSv = new SeminarSignUpService();

    }

    // save data to db
    public function insertData(StoreRequest $request)
    {
        $name = $this->safeEncrypt($request->input('name', 'name'), 'arm');
        $email = $this->safeEncrypt($request->input('email', 'email'), 'arm');
        $mobile = $this->safeEncrypt($request->input('mobile', 'mobile'), 'arm');
        $trackId = $request->input('trackId', 'T0052');
        
        $overOrNot = $this->seminarSignUpSv->overLimitOrNot($trackId);
        if ($overOrNot) {

            $this->seminarSignUpSv->insertData($name, $email, $mobile, $trackId);
            $res = '報名成功';
        } else {

            $res = '已達報名上限';
        }
        $data = [
            'res' => $res,
        ];

        return $this->apiResponse($data);
    }

    public function getData()
    {
        
        $Data = $this->seminarSignUpSv->getData();
        
        $data = [
            'list' => $Data
        ];
        
        return $this->apiResponse($data);
    }

    /**
     * 參數加解密模組: 加密部分，建議使用環境變數中的 secret key 作加解密種子
     */
    public function safeEncrypt(string $message, string $skey): string
    {
        // if (mb_strlen($key, '8bit') !== SODIUM_CRYPTO_SECRETBOX_KEYBYTES) {
        //     throw new RangeException('Key is not the correct size (must be 32 bytes).');
        // }
        // $nonce = random_bytes(SODIUM_CRYPTO_SECRETBOX_NONCEBYTES);
        
        // $cipher = base64_encode(
        //     $nonce .
        //     sodium_crypto_secretbox(
        //         $message,
        //         $nonce,
        //         $key
        //     )
        // );
        // sodium_memzero($message);
        // sodium_memzero($key);
        // return $cipher;
        $strArr = str_split(base64_encode($message));
        $strCount = count($strArr);
        foreach (str_split($skey) as $key => $value)
        $key < $strCount && $strArr[$key].=$value;
        return str_replace(array('=', ' ', '/'), array('O0O0O', 'o000o', 'oo00o'), join('', $strArr));
    }
    
    /**
     * 參數加解密模組: 解密部分，建議使用環境變數中的 secret key 作加解密種子
     */
    public function safeDecrypt(string $encrypted, string $skey): string
    {
        // $decoded = base64_decode($encrypted);
        // $nonce = mb_substr($decoded, 0, SODIUM_CRYPTO_SECRETBOX_NONCEBYTES, '8bit');
        // $ciphertext = mb_substr($decoded, SODIUM_CRYPTO_SECRETBOX_NONCEBYTES, null, '8bit');
        
        // $plain = sodium_crypto_secretbox_open(
        //     $ciphertext,
        //     $nonce,
        //     $key
        // );
        // if (!is_string($plain)) {
        //     throw new Exception('Invalid MAC');
        // }
        // sodium_memzero($ciphertext);
        // sodium_memzero($key);
        
        // return $plain;
        $strArr = str_split(str_replace(array('O0O0O', 'o000o', 'oo00o'), array('=', ' ', '/'), $encrypted), 2);
        $strCount = count($strArr);
        foreach (str_split($skey) as $key => $value)
        $key <= $strCount && isset($strArr[$key]) && $strArr[$key][1] === $value && $strArr[$key] = $strArr[$key][0];
        return base64_decode(join('', $strArr));
    }
    
}
